Summary:          MirBSD enhanced version of the Korn Shell
Name:             mksh
Version:          59c
Release:    1%{?dist}
License:          MirOS AND ISC
URL:              https://www.mirbsd.org/mksh.htm
Source0:          https://www.mirbsd.org/MirOS/dist/mir/%{name}/%{name}-R%{version}.tgz
Source1:          dot-mkshrc
Source2:          rtchecks.expected
Conflicts:        filesystem < 3
Provides:         /bin/ksh
Provides:         /bin/lksh
Provides:         /bin/mksh
Provides:         /bin/rksh
Provides:         /bin/rlksh
Provides:         /bin/rmksh
Requires(post):   grep
Requires(post):   %{_sbindir}/alternatives
Requires(preun):  %{_sbindir}/alternatives
Requires(postun): sed
BuildRequires:    gcc
BuildRequires:    util-linux
BuildRequires:    ed
BuildRequires:    perl-interpreter
BuildRequires:    perl(Getopt::Std)
BuildRequires:    sed

%description
mksh is the MirBSD enhanced version of the Public Domain Korn shell (pdksh),
a bourne-compatible shell which is largely similar to the original AT&T Korn
shell. It includes bug fixes and feature improvements in order to produce a
modern, robust shell good for interactive and especially script use, being a
bourne shell replacement, pdksh successor and an alternative to the C shell.

%prep
%setup -q -n %{name}

cat >rtchecks <<'EOF'
typeset -i sari=0
typeset -Ui uari=0
typeset -i x=0
print -r -- $((x++)):$sari=$uari. #0
let --sari --uari
print -r -- $((x++)):$sari=$uari. #1
sari=2147483647 uari=2147483647
print -r -- $((x++)):$sari=$uari. #2
let ++sari ++uari
print -r -- $((x++)):$sari=$uari. #3
let --sari --uari
let 'sari *= 2' 'uari *= 2'
let ++sari ++uari
print -r -- $((x++)):$sari=$uari. #4
let ++sari ++uari
print -r -- $((x++)):$sari=$uari. #5
sari=-2147483648 uari=-2147483648
print -r -- $((x++)):$sari=$uari. #6
let --sari --uari
print -r -- $((x++)):$sari=$uari. #7
(( sari = -5 >> 1 ))
((# uari = -5 >> 1 ))
print -r -- $((x++)):$sari=$uari. #8
(( sari = -2 ))
((# uari = sari ))
print -r -- $((x++)):$sari=$uari. #9
EOF

%build
CFLAGS="$RPM_OPT_FLAGS -DMKSH_DISABLE_EXPERIMENTAL" LDFLAGS="$RPM_LD_FLAGS" sh Build.sh -r
cp test.sh test_mksh.sh
HAVE_PERSISTENT_HISTORY=0; export HAVE_PERSISTENT_HISTORY
CFLAGS="$RPM_OPT_FLAGS -DMKSH_DISABLE_EXPERIMENTAL" LDFLAGS="$RPM_LD_FLAGS" sh Build.sh -L -r
cp -f test.sh test_lksh.sh
./mksh FAQ2HTML.sh

%install
install -D -m 755 %{name} %{buildroot}%{_bindir}/%{name}
install -D -m 755 lksh %{buildroot}%{_bindir}/lksh
install -D -m 644 %{name}.1 %{buildroot}%{_mandir}/man1/%{name}.1
install -D -m 644 lksh.1 %{buildroot}%{_mandir}/man1/lksh.1
install -D -p -m 644 dot.mkshrc %{buildroot}%{_sysconfdir}/mkshrc
install -D -p -m 644 %{SOURCE1} %{buildroot}%{_sysconfdir}/skel/.mkshrc
ln -s %{name} %{buildroot}%{_bindir}/rmksh
ln -s lksh %{buildroot}%{_bindir}/rlksh
ln -s %{name}.1 %{buildroot}%{_mandir}/man1/rmksh.1
ln -s lksh.1 %{buildroot}%{_mandir}/man1/rlksh.1
touch %{buildroot}{%{_bindir}/{ksh,rksh},%{_mandir}/man1/{ksh,rksh}.1}

%check
./mksh rtchecks >rtchecks.got 2>&1
if ! cmp --quiet rtchecks.got %{SOURCE2}
then
  echo "rtchecks failed"
  diff -Naurp %{SOURCE2} rtchecks.got
  exit 1
fi

for tf in test_mksh.sh test_lksh.sh
do
  echo > test.wait
  script -qc "./$tf"' -v; x=$?; rm -f test.wait; exit $x'
  maxwait=0
  while test -e test.wait; do
    sleep 1
    maxwait=$(expr $maxwait + 1)
    test $maxwait -lt 900 || break
  done
done

%post
for d in /bin %{_bindir}; do
  for s in ksh %{name} rksh rmksh; do
    grep -q '^'"$d/$s"'$' %{_sysconfdir}/shells 2>/dev/null || echo "$d/$s" >> /etc/shells
  done
done

%{_sbindir}/alternatives --install %{_bindir}/ksh ksh %{_bindir}/%{name} 10 \
  --slave %{_bindir}/rksh rksh %{_bindir}/%{name} \
  --slave %{_mandir}/man1/ksh.1.gz ksh-man %{_mandir}/man1/%{name}.1.gz \
  --slave %{_mandir}/man1/rksh.1.gz rksh-man %{_mandir}/man1/%{name}.1.gz

%preun
if [ $1 -eq 0 ]; then
  %{_sbindir}/alternatives --remove ksh %{_bindir}/%{name}
fi

%postun
for d in /bin %{_bindir}; do
  for s in ksh %{name} rksh rmksh; do
    [ ! -x "$d/$s" ] && sed -e 's@^'"$d/$s"'$@POSTUNREMOVE@' -e '/^POSTUNREMOVE$/d' -i %{_sysconfdir}/shells
  done
done

%files
%doc dot.mkshrc FAQ.htm
%ghost %{_bindir}/ksh
%{_bindir}/lksh
%{_bindir}/%{name}
%ghost %{_bindir}/rksh
%{_bindir}/rlksh
%{_bindir}/rmksh
%config(noreplace) %{_sysconfdir}/mkshrc
%config(noreplace) %{_sysconfdir}/skel/.mkshrc
%ghost %{_mandir}/man1/ksh.1*
%{_mandir}/man1/lksh.1*
%{_mandir}/man1/%{name}.1*
%ghost %{_mandir}/man1/rksh.1*
%{_mandir}/man1/rlksh.1*
%{_mandir}/man1/rmksh.1*

%changelog
* Fri Aug 02 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 59c-1
- initial build
